# TokenServer
This Rails plugin creates a jwt (https://jwt.io) token. This created token is compatible with the Report-Server.

## Usage

## Installation
Add this line to your application's Gemfile:

```ruby
gem 'token_server'
```

Add this engine to your Rails project and mount it by adding the following line into your routes.rb:
```ruby
mount TokenServer::Engine => "token/"
```
Now you can access the token server via get "token/" authenticated with basic-authentication.

Now you need to configure the account class that you will use (you have to implement it by yourself!). The account class accesses the user and his rights in your system and adds all necessary data (roles and rights) into the User object that it returns:

Add this to your environment-files (like produrction.rb)
```ruby
TokenServer::account_class = "YOUR-ACCOUNT-CLASS"
TokenServer::header_param_keys = ["OPTIONAL-HEADER-PARAMS", ...]
config.token_secret = 'test_key'
```
The *account\_class* defines the class that implements how to authenticate into the server layer and which information needs to be added into the token.

The *token\_secret* is the shared secret used by the Report-Server and the token server[^1] for signing the token.

Basic-Authentication is used to authenticate against the token server. Due to the fact that some service layer might need additional information (like a client id), you can define addtional header params that should be passed via *header\_param\_keys* to the *acount\_class*.

This *YOUR-ACCOUNT-CLASS* is the name of the class that should look like this:

```ruby
module MyNameSpace
    class Account
        include ActiveModel::Model

        # Select different account models if necessary regarding to the sub-domain
        # (not used yet)
        def self.find_by(url_name: nil)
            Account.new
        end

        # Find user and check password. Return a user object that
        # contains the right information.
        # user: The user identification used for login (via Basic-Authentication)
        # password: The password (via Basic-Authentication)
        # meta: Additional information defined by header_param_keys
        def authenticate(user, password, meta = {})
            # 1. Do authentication against your service layer
            # 2. Obtain everything you need to pass to the requesting instance and put it into the credentials_hash
            # 3. Check whether this user should have admin rights in the report server and use this informaiton for is_plugin_manager
    
            # Now create the token
            ::TokenServer::User.new(user_id: user, password: password, is_plugin_manager: true, credentials_hash: {})
        end
    end
end
```

Now you need to configure the built in knock gem (see: https://github.com/eilers/knock). Please copy this file from the Report-Server configuration: https://src2.imhosting.de/vrg/Report-Server/blob/master/config/initializers/knock.rb

This file defines everything necessary to create the correct token.

Now execute:
```bash
$ bundle
```

Or install it yourself as:
```bash
$ gem install token_server
```

## Contributing
Contribution directions go here.

## License
The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).

[^1]: #TODO: Discuss whether it should be in the environment and not in the configuration..